# -*- coding: utf-8 -*-
from operator import itemgetter
from .validip import is_valid_ip

def dailyTopUrl(d, owndomain, omit):
    topurlcnt = 0
    topurl = {}
    for k, v in sorted(d['v0001']['days'].items(), key=itemgetter(0), reverse=True):
        # consider only days:
        if len(k) < 8:
            continue
        if any(oelm in k for oelm in omit):  # don not show parts of url 
            continue
        # show at max 31 days:
        if topurlcnt >= 31:
            break
        topurlcnt += 1

        # top 10 source domains:
        if 'topUrl' in d['v0001']['days'][k]['user']:
            for tk, tv in d['v0001']['days'][k]['user']['topUrl'].items():
                # check if url is blocked for display:
                if any(oelm in tk for oelm in omit):  # don not show parts of url 
                    continue
                if tk not in topurl:
                    topurl[tk] = 0
                topurl[tk] += tv
   
    if len(topurl) == 0:
        return ''
    
    h = "\n" + '<div class="col-md-12 col-lg-6 col-xxl-6">'
    h += '<h3>Top 10 URL</h3>'
    h += '<p>User hits for the last ' + str(topurlcnt) + ' days by internal URL on ' + owndomain + ':'
    h += '<table class="table">'
    h += '<thead><tr><th scope="col" style="text-align: left">Rank</th><th scope="col" style="text-align: left">URL</th><th scope="col" style="text-align: right">Hits</th></tr></thead>'
    i = 1
    for k, v in sorted(topurl.items(), key=itemgetter(1), reverse=True):
        if not k.endswith('.css') and not k.endswith('.json') and not k.endswith('.ico'):
            h += '<tr><td>' + str(i) + '.</td><td>' + str(k) + '</td><td style="text-align: right">' + str(format(v, ',')) + '</td></tr>'
            i += 1
            if i > 10:
                break
    h += '</table>'
    h += '</div>'  + "\n"   # end of col and card
 
    return h



# -*- coding: utf-8 -*-
from operator import itemgetter
from .validip import is_valid_ip

def monthlyTopCountries(d, owndomain, omit):
    tcountries = {}
    tcnt = 0
    for k, v in sorted(d['v0001']['days'].items(), key=itemgetter(0), reverse=True):
        # consider only days:
        if len(k) > 6:
            continue
        # show at max 31 days:
        if tcnt >= 31:
            break
        tcnt += 1

        if 'countryHits' in d['v0001']['days'][k]['user']:
            for co,cv in d['v0001']['days'][k]['user']['countryHits'].items():
                if co not in tcountries:
                    tcountries[co] = 0
                tcountries[co] += cv
           
    if len(tcountries) == 0:
        return ''
    
    h = "\n" + '<div class="col-md-12 col-lg-6 col-xxl-6 pt-3">'
    h += '<h3>Top 10 Countries - Long Term</h3>'
    h += '<p>User hits from countries on ' + owndomain + ':'
    h += '<table class="table">'
    h += '<thead><tr><th scope="col" style="text-align: left">Rank</th><th scope="col" style="text-align: left">Country</th><th scope="col" style="text-align: right">Hits</th></tr></thead>'
    i = 1
    for k, v in sorted(tcountries.items(), key=itemgetter(1), reverse=True):
        if not k.endswith('.css') and not k.endswith('.json') and not k.endswith('.ico'):
            h += '<tr><td>' + str(i) + '.</td><td>' + str(k) + '</td><td style="text-align: right">' + str(format(v, ',')) + '</td></tr>'
            i += 1
            if i > 10:
                break
    h += '</table>'
    h += '</div>'  + "\n"   # end of col and card
 
    return h



# -*- coding: utf-8 -*-
from operator import itemgetter
import ipaddress

def monthlyHitsVisitsChart(d, owndomain, omit):
    rdata = []
    sdata = []
    vdata = []
    mcount = 0
    startPeriod = None
    endPeriod = None
    for k, v in sorted(d['v0001']['days'].items(), key=itemgetter(0), reverse=True):
        # omit days:
        if len(k) > 6:
            continue
        if mcount > 31:
            break
        mcount += 1
        if 'desktop' in d['v0001']['days'][k]['user']['deviceHits']:
            sdata.append({'d': k, 't': 'desktop', 'c': d['v0001']['days'][k]['user']['deviceHits']['desktop']})
        else:
            sdata.append({'d': k, 't': 'desktop', 'c': 0})
        if 'mobile' in d['v0001']['days'][k]['user']['deviceHits']:
            sdata.append({'d': k, 't': 'mobile', 'c': d['v0001']['days'][k]['user']['deviceHits']['mobile']})
        else:
            sdata.append({'d': k, 't': 'mobile', 'c': 0})
        if 'tablet' in d['v0001']['days'][k]['user']['deviceHits']:
            sdata.append({'d': k, 't': 'tablet', 'c': d['v0001']['days'][k]['user']['deviceHits']['tablet']})
        else:
            sdata.append({'d': k, 't': 'tablet', 'c': 0})

        # visits:
        if 'visits' in d['v0001']['days'][k]['user']:
            vdata.append({'d': k, 'c': d['v0001']['days'][k]['user']['visits']})
        else:
            vdata.append({'d': k, 'c': 0})

        # robots
        if 'robotHits' in d['v0001']['days'][k]['robot']:
            rdata.append({'d': k, 'c': d['v0001']['days'][k]['robot']['robotHits']})
        else:
            rdata.append({'d': k, 'c': 0})

        # record start / end period to be displayed on chart:
        if startPeriod is None or k < startPeriod:
            startPeriod = k
        if endPeriod is None:
            endPeriod = k

    # first run / first day:
    if len(sdata) == 0:
        return ''
        
    # d3js horizontal bubble char in case results are available
    h = "\n\n"
    h += '<div class="col-md-12 col-lg-12 col-xxl-12 pt-4">'
    h += '<h3>User Hits and Visits - Long Term</h3>'
    #h += '<p>User hits and visits on ' + owndomain + ' from ' + startPeriod + ' to ' + endPeriod + ':</p>'
    
    h += "<p>User hits and visits on " + owndomain + " from <script>document.write(DT2Locale('" + startPeriod + "01'));</script> to "
    h += "<script>document.write(DT2Locale('" + endPeriod + "'+ daysInMonth('" + endPeriod[0:3] + "','" + endPeriod[4:5] + "')));</script>:</p>"
    h += '<div id="mhvchart"></div>' + "\n"
    h += '<script type="module">' + "\n"
    h += 'const sdata = ' + str(sdata) + ';' + "\n"
    h += 'const vdata = ' + str(vdata) + ';' + "\n"
    h += 'const rdata = ' + str(rdata) + ';' + "\n"

    h += 'const groupedData = d3.rollup(sdata, v => Object.fromEntries(v.map(d => [d.t, d.c])),d => d.d);'  + "\n"

    # Convert to array with missing categories filled as 0
    h += 'const dates = Array.from(groupedData.keys()).sort();'  + "\n"
    h += 'const categories = ["desktop", "mobile", "tablet"];'  + "\n"

    h += 'const transformedData = dates.map(d => {'
    h += 'let entry = { d };'
    h += 'let values = groupedData.get(d) || {};'  # Ensure it's always an object
    h += 'categories.forEach(c => entry[c] = values[c] ?? 0);'  # Use Nullish coalescing to handle undefined
    h += 'return entry;'
    h += '});'  + "\n"

    # Sort the categories within each date by count in ascending order
    h += 'transformedData.forEach(entry => {'
    h += 'const sortedCategories = categories.slice().sort((a, b) => entry[a] - entry[b]);'
    h += 'const sortedEntry = {};'
    h += 'sortedCategories.forEach(c => sortedEntry[c] = entry[c]);'
    h += 'Object.assign(entry, sortedEntry);'
    h += '});'  + "\n"

    # Stack generator
    h += 'const stack = d3.stack().keys(categories);'  + "\n"
    
    # // Manually generate stack series with per-bar sorting
    h += 'const series0 = stack(transformedData);'  + "\n"
    h += 'const series = categories.map(c => ({ key: c, values: [] }));'
    h += 'for (let i = 0; i < transformedData.length; i++) {'
    h += 'const entry = transformedData[i];'
    h += 'const sortedKeys = categories.slice().sort((a, b) => entry[a] - entry[b]);'
    h += 'let y0 = 0;'
    h += 'sortedKeys.forEach(key => {'
    h += 'const value = entry[key];'
    h += 'const y1 = y0 + value;' 
    # Find the matching series object and add this datum
    h += 'const s = series.find(s => s.key === key);'
    h += 's.values[i] = [y0, y1, entry, key];' # store additional info for tooltip
    h += 'y0 = y1;'
    h += '});'
    h += '}' + "\n"
        
    # getting max from series,vdata and sdata:
    h += 'let yMax = Math.max('
    h += 'd3.max(series, s => d3.max(s.values, v => v[1])),'
    h += 'd3.max(vdata, d => d.c),'
    h += 'd3.max(rdata, d => d.c)'
    h += ');' 
    # round up
    h += "const rm = yMax % 1000;"
    h += "yMax += 1000 - rm;" + "\n"
    
    h += 'const container = document.getElementById("mhvchart");'
    h += 'const { width } = container.getBoundingClientRect();'
    h += 'const height = width * 0.5;' + "\n"

    h += 'const margins = { top: 20, right: 20, bottom: 50, left: 40 };'
    h += 'const x = d3.scaleBand().domain(dates).range([margins.left, width - margins.right]).padding(0.1);'  + "\n"
    
    h += 'const y = d3.scaleSymlog()'
    h += '.domain([0.1, yMax])'   # Log scale cannot have 0
    h += '.range([height - margins.bottom, margins.top])'
    h += '.nice();'  + "\n"

    h += 'const color = d3.scaleOrdinal().domain(categories).range(["#42f5aa", "#42c5f5", "#f5a742"]);' + "\n"

    h += 'const svg = d3.select("#mhvchart")'
    h += '.append("svg")'
    h += '.attr("width", width)'
    h += '.attr("height", height)'
    h += '.attr("viewBox", [0, 0, width, height])'
    h += '.attr("style", "font: 10px sans-serif;");'  + "\n"

    # Add bars
    h += 'svg.append("g")'
    h += '.selectAll("g")'
    h += '.data(series)'
    h += '.join("g")'
    h += '.attr("fill", d => color(d.key))'
    h += '.selectAll("rect")'
    h += '.data(d => d.values)'
    h += '.join("rect")'
    h += '.attr("x", d => x(d[2].d))'
    h += '.attr("y", d => y(Math.max(1, isNaN(d[1]) ? 1 : d[1])))'
    h += '.attr("height", d => Math.max(0, Math.abs(y(d[0]) - y(d[1]))))'
    h += '.attr("width", x.bandwidth());' + "\n"

    h += 'const visitline = d3.line()'
    h += '.x(d => x(d.d) + x.bandwidth()/2 )'
    h += '.y(d => y(d.c));' + "\n"

    # Visits
    h += 'svg.append("path")'
    h += '.datum(vdata)' # Bind data properly
    h += '.attr("fill", "none")'
    h += '.attr("stroke", "red")'
    h += '.attr("stroke-width", 2)'
    h += '.attr("d", visitline);' + "\n"

    # Robots
    h += 'svg.append("path")'
    h += '.datum(rdata)' # Bind data properly
    h += '.attr("fill", "none")'
    h += '.attr("stroke", "lightgrey")'
    h += '.attr("stroke-width", 2)'
    h += '.attr("d", visitline);' + "\n"

    # X-Axis
    h += 'svg.append("g")'
    h += '.attr("transform", `translate(0,${height - margins.bottom})`)'
    h += '.call(d3.axisBottom(x).tickSizeOuter(0))'
    h += '.selectAll("text")'
    h += '.attr("transform", "rotate(45)")'
    h += '.style("text-anchor", "start");' + "\n"

    # Customize y-axis ticks and labels
    h += 'const yAxis = d3.axisLeft(y)'
    h += '.ticks(12)'  # Specify the number of ticks you want
    h += '.tickFormat(d => `${d} units`);' + "\n" #  Format the labels to include "units"

    # Y-Axis
    h += 'const tickValues = [1, 10, 50, 100, 200, 500,1000,2000,5000,10000,yMax];' 
    h += 'svg.append("g")'
    h += '.attr("transform", `translate(${margins.left},0)`)'
    h += '.call(d3.axisLeft(y)'
    h += '.tickValues(tickValues)'
    h += '.tickSize(-width + margins.left + margins.right)'
    h += '.tickFormat(d3.format("~s")))' # Format numbers neatly
    h += '.selectAll("line")'
    h += '.style("stroke", "lightgrey")'  # Set gridline color
    h += '.style("stroke-opacity", 0.5);'  + "\n"

    # Create a tooltip div
    h += 'const tooltip = d3.select("#mhvchart").append("div")'
    h += '.attr("class", "custom-tooltip")'
    h += '.style("position", "absolute")'
    h += '.style("background", "rgba(255, 255, 255, 0.95)")'
    h += '.style("box-shadow", "0 2px 8px rgba(0,0,0,0.15)")'
    h += '.style("padding", "8px 12px")'
    h += '.style("border-radius", "6px")'
    h += '.style("font-size", "12px")'
    h += '.style("pointer-events", "none")'
    h += '.style("transition", "opacity 0.3s ease")'
    h += '.style("opacity", 0);'  + "\n"

    # Add tooltip functionality to bars
    h += 'svg.selectAll("rect")'
    h += '.on("mouseover", (event, d) => { tooltip.style("visibility", "visible").style("opacity", 1); })'
    h += '.on("mousemove", (event, d) => {'
    h += 'tooltip.html(`Date: ${d[2].d}<br>Category: ${d[3]}<br>Count: ${d[1] - d[0]}`)'
    h += '.style("top", `${event.pageY - 10}px`).style("left", `${event.pageX + 10}px`);'
    h += '})'
    h += '.on("mouseleave", () => { tooltip.style("visibility", "hidden").style("opacity", 0); });' + "\n"

    # Add legend
    h += 'const legend = svg.append("g")'
    # Position the legend in the upper left corner:
    h += '.attr("transform", `translate(${margins.left + 6}, ${margins.top + 6})`);'

    h += 'legend.selectAll(".legend-item")'
    h += '.data(categories)'
    h += '.enter().append("g")'
    h += '.attr("class", "legend-item")'
    h += '.attr("transform", (d, i) => `translate(${i * 100}, 0)`)'
    h += '.each(function(d, i) {'
    h += 'const item = d3.select(this);'
    h += 'item.append("rect")'
    h += '.attr("width", 18)'
    h += '.attr("height", 18)'
    h += '.attr("fill", color(d));'
    h += 'item.append("text")'
    h += '.attr("x", 24)'
    h += '.attr("y", 9)'
    h += '.attr("dy", "0.35em")'
    h += '.text(d);'
    h += '});'  + "\n" 
    
    #h += 'console.log("monthlyHits->Transformed Data:", transformedData);'
    #h += 'console.log("monthlyHits->Stacked Data:", stack(transformedData));'

    h += 'function addPoints(data, color, label) {'
    h += 'svg.selectAll(`.point-${color}`)'
    h += '.data(data).enter()'
    h += '.append("circle")'
    h += '.attr("class", `point-${color}`)'
    h += '.attr("cx", d => x(d.d) + x.bandwidth() / 2)'
    h += '.attr("cy", d => y(d.c))'
    h += '.attr("r", 4)'
    h += '.attr("fill", color)'
    h += '.attr("stroke", "white")'
    h += '.attr("stroke-width", 1)'
    h += '.on("mouseover", (event, d) => { tooltip.style("opacity", 1).style("visibility","visible"); })'
    h += '.on("mousemove", (event, d) => {'
    h += 'const date = d.d;'
    h += 'const vdataItem = vdata.find(item => item.d === date);'
    h += 'const rdataItem = rdata.find(item => item.d === date);'
    h += 'tooltip.html(`'
    h += '   <strong>${label}</strong><br>'
    h += '   <strong>Count: ${d3.format(",")(d.c)}</strong><br>'
    h += '   Date: ${date}<br>'
    h += '   Visits: ${vdataItem ? vdataItem.c : "N/A"}<br>'
    h += '   Requests: ${rdataItem ? rdataItem.c : "N/A"}'
    h += '`)'
    h += '.style("top",  `${event.pageY - 40}px`)'
    h += '.style("left", `${event.pageX + 15}px`);'
    h += '})'
    h += '.on("mouseleave", () => {tooltip.style("opacity", 0); });'
    h += '}' + "\n"

    h += 'addPoints(vdata, "red", "Visits");'  + "\n"
    h += 'addPoints(rdata, "lightgrey", "Requests");'  + "\n"

    h += "</script>"
    h += '</div>' + "\n"
    return h



from copy import deepcopy
from datetime import datetime, timedelta
from operator import itemgetter
import json

def sumDay2MonthV0001(d, statfile, verbosity):
    
    # accumulate previous months:
    # 2024.08.02 => 2024.06
    # 2024.08.30 => 2024.06
    # 2024.08.31 => 2024.07

    # take timelastrec as based for sum - reason: regression tests   
    sumUntilDay = datetime.strptime(d['timelastrec'],"%Y%m%d%H%M%S") - timedelta(days=62)
    sumTill = sumUntilDay.strftime("%Y%m")
    print("sumDay2MonthV0001: accumulate including month: "  + sumTill)   # 202312
    
    # iterate over days to accumulate months:
    dtmp = deepcopy(d)
    for x in dtmp['v0001']['days']:
        # if date has a day:
        if len(x) > 6:
            recMonth = x[0:6]
            if recMonth <= sumTill:
                if verbosity == "99":
                    print("sumDay2MonthV0001: recMonth : " + recMonth + ",  day is: " + x)
                if recMonth not in d['v0001']['days']:
                    d['v0001']['days'][recMonth] = {}
                    d['v0001']['days'][recMonth]['user'] = {}
                    d['v0001']['days'][recMonth]['user']['visits'] = 0;
                    d['v0001']['days'][recMonth]['user']['bytesSent']   = 0;
                    d['v0001']['days'][recMonth]['user']['countryHits'] = {}
                    d['v0001']['days'][recMonth]['user']['deviceHits'] = {}
                    d['v0001']['days'][recMonth]['user']['deviceHits']['mobile']  = 0 
                    d['v0001']['days'][recMonth]['user']['deviceHits']['tablet']  = 0 
                    d['v0001']['days'][recMonth]['user']['deviceHits']['desktop'] = 0 

                    d['v0001']['days'][recMonth]['user']['topUrl'] = {}
                    d['v0001']['days'][recMonth]['user']['serverResponseCode'] = {}
                    d['v0001']['days'][recMonth]['robot'] = {}
                    d['v0001']['days'][recMonth]['robot']['bytesSent']   = 0;
                    d['v0001']['days'][recMonth]['robot']['robotHits']   = 0;
                    d['v0001']['days'][recMonth]['quality'] = {}

                # always check, if structure exists as we might start within the month (not on the first day of month)
                if 'visits' in dtmp['v0001']['days'][x]['user']:
                    d['v0001']['days'][recMonth]['user']['visits']     += dtmp['v0001']['days'][x]['user']['visits']
                if 'bytesSent' in dtmp['v0001']['days'][x]['user']:
                    d['v0001']['days'][recMonth]['user']['bytesSent']  += int(dtmp['v0001']['days'][x]['user']['bytesSent'])

                if 'countryHits' in dtmp['v0001']['days'][x]['user']:
                    for ck,cv in dtmp['v0001']['days'][x]['user']['countryHits'].items():
                        if verbosity == "99":
                            print("sumDay2Month: country: " + ck + ": " + str(cv) )
                            print("sumDay2Month: countries" + str(dtmp['v0001']['days'][recMonth]['user']['countryHits']))
                        if ck in d['v0001']['days'][recMonth]['user']['countryHits']:
                            d['v0001']['days'][recMonth]['user']['countryHits'][ck] += cv
                        else:
                            d['v0001']['days'][recMonth]['user']['countryHits'][ck] = cv;
                
                if 'deviceHits' in  dtmp['v0001']['days'][x]['user']:   
                    d['v0001']['days'][recMonth]['user']['deviceHits']['mobile']  += dtmp['v0001']['days'][x]['user']['deviceHits']['mobile']
                    d['v0001']['days'][recMonth]['user']['deviceHits']['tablet']  += dtmp['v0001']['days'][x]['user']['deviceHits']['tablet']
                    d['v0001']['days'][recMonth]['user']['deviceHits']['desktop'] += dtmp['v0001']['days'][x]['user']['deviceHits']['desktop']
                
                if 'robot' in dtmp['v0001']['days'][x]:
                    if 'robotHits' in dtmp['v0001']['days'][x]['robot']:
                        d['v0001']['days'][recMonth]['robot']['robotHits'] += dtmp['v0001']['days'][x]['robot']['robotHits']
                    if 'bytesSent' in dtmp['v0001']['days'][x]['robot']:
                        d['v0001']['days'][recMonth]['robot']['bytesSent'] += int(dtmp['v0001']['days'][x]['robot']['bytesSent'])

                # Server Response Code:
                if 'serverResponseCode' in dtmp['v0001']['days'][x]['user']:
                    for ck,cv in dtmp['v0001']['days'][x]['user']['serverResponseCode'].items():
                        #print("sumV0001:  Responsecode: " + ck + ": " + str(cv) )

                        #print("sumV0001 responseCode: " + str(d['v0001']['days'][recMonth]['user']['serverResponseCode']))
                        if ck in d['v0001']['days'][recMonth]['user']['serverResponseCode']:
                            d['v0001']['days'][recMonth]['user']['serverResponseCode'][ck] += cv
                        else:
                            d['v0001']['days'][recMonth]['user']['serverResponseCode'][ck] = cv;

                # topurl:
                if 'topUrl' in dtmp['v0001']['days'][x]['user']:
                    for ck,cv in dtmp['v0001']['days'][x]['user']['topUrl'].items():
                        #print("sum: topurl: " + ck + ": " + str(cv) )   
                        # sum: topurl: /kl/view/325fbfbf-cf58-400e-abc4-4173e7b015e4/: 2

                        #print("sum topurl: " + str(d['v0001']['days'][recMonth]['user']['topUrl']))
                        # sum topurl: {'/kl/view/f95c0c51-5f00-42f6-ae4c-50abb30ed665/': 2,
                        #              '/kl/view/973a9773-2f06-4f8c-a4b6-ac785b96c990/' : 1,
                        
                        if ck in d['v0001']['days'][recMonth]['user']['topUrl']:
                            d['v0001']['days'][recMonth]['user']['topUrl'][ck] += cv
                        else:
                            d['v0001']['days'][recMonth]['user']['topUrl'][ck] = cv;
                            
                # quality:
                if 'quality' in dtmp['v0001']['days'][x]:
                    for ck,cv in dtmp['v0001']['days'][x]['quality'].items():
                        if ck in d['v0001']['days'][recMonth]['quality']:
                            d['v0001']['days'][recMonth]['quality'][ck]['count'] += 1
                        else:
                            d['v0001']['days'][recMonth]['quality'][ck] = cv

                # nav   - nav is stored on the last day of month
                if 'nav' in dtmp['v0001']['days'][x]['user']:
                    if 'nav' not in d['v0001']['days'][recMonth]['user']:
                        d['v0001']['days'][recMonth]['user']['nav'] = []
                    # take to top 20 external:
                    cnt = 0
                    for e in sorted(d['v0001']['days'][x]['user']['nav'], key=itemgetter('c'), reverse=True):
                        if 'p' not in e or e['p'] != 'e':  # not external nav
                            continue
                        if verbosity == "99":
                            print('summDay2MonthV0001: nav: ' + str(x) + ': ' + str(e))
                        d['v0001']['days'][recMonth]['user']['nav'].append(e)
                        cnt += 1
                        if cnt > 20:
                            break

                    # take to top 50 internal:
                    cnt = 0
                    for e in sorted(d['v0001']['days'][x]['user']['nav'], key=itemgetter('c'), reverse=True):
                        if 'p' in e and e['p'] == 'e':  # omit external
                            continue
                        if verbosity == "99":
                            print('summDay2MonthV0001: nav: ' + str(x) + ': ' + str(e))
                        d['v0001']['days'][recMonth]['user']['nav'].append(e)
                        cnt += 1
                        if cnt > 50:
                            break

                # language - optional KPI:
                if 'language' in dtmp['v0001']['days'][x]['user']:
                    if 'language' not in d['v0001']['days'][recMonth]['user']:
                        d['v0001']['days'][recMonth]['user']['language'] = {}
                    for nk, nv in sorted(dtmp['v0001']['days'][x]['user']['language'].items(), key=itemgetter(1), reverse=True):
                        if nk in d['v0001']['days'][recMonth]['user']['language']:
                            d['v0001']['days'][recMonth]['user']['language'][nk] += nv
                        else:
                            d['v0001']['days'][recMonth]['user']['language'][nk] = nv

                # performance - optional KPI:
                if 'performance' in dtmp['v0001']['days'][x]:
                    if 'performance' not in d['v0001']['days'][recMonth]:
                        d['v0001']['days'][recMonth]['performance'] = {}
                        
                    if 'cache' in dtmp['v0001']['days'][x]['performance']:
                        d['v0001']['days'][recMonth]['performance']['cache'] = {}
                        for nk, nv in sorted(dtmp['v0001']['days'][x]['performance']['cache'].items(), key=itemgetter(1), reverse=True):
                            if nk in d['v0001']['days'][recMonth]['performance']['cache']:
                                d['v0001']['days'][recMonth]['performance']['cache'][nk] += nv
                            else:
                                d['v0001']['days'][recMonth]['performance']['cache'][nk] = nv
                                
                    if 'response_time' in dtmp['v0001']['days'][x]['performance']:
                        d['v0001']['days'][recMonth]['performance']['response_time'] = {}
                        for nk, nv in sorted(dtmp['v0001']['days'][x]['performance']['response_time'].items(), key=itemgetter(1), reverse=True):
                            if nk in d['v0001']['days'][recMonth]['performance']['response_time']:
                                d['v0001']['days'][recMonth]['performance']['response_time'][nk] += nv
                            else:
                                d['v0001']['days'][recMonth]['performance']['response_time'][nk] = nv
                
                del d['v0001']['days'][x]

    # compress by month
    for x in d['v0001']['days']:
        if len(x) == 6:
            if 'countryHits' in d['v0001']['days'][x]['user']:
                tmpCountryHits = dict(sorted(d['v0001']['days'][x]['user']['countryHits'].items(), key=lambda item: item[1], reverse=True))
                i = 0
                d['v0001']['days'][x]['user']['countryHits'] = {}
                for k,v in tmpCountryHits.items():
                    i += 1
                    if i >= 24:
                        break
                    d['v0001']['days'][x]['user']['countryHits'][k] = v
                del tmpCountryHits
                
            # take only 24 TopUrl in a month:
            if 'topUrl' in d['v0001']['days'][x]['user']:
                tmpTopUrl = dict(sorted(d['v0001']['days'][x]['user']['topUrl'].items(), key=lambda item: item[1], reverse=True))
                i = 0
                d['v0001']['days'][x]['user']['topUrl'] = {}
                for k,v in tmpTopUrl.items():
                    i += 1
                    if i >= 24:
                        break
                    d['v0001']['days'][x]['user']['topUrl'][k] = v
                del tmpTopUrl
                
            # take only 24 navigations per month:
            if 'navigation' in d['v0001']['days'][x]['user']:
                tmpNav = dict(sorted(d['v0001']['days'][x]['user']['navigation'].items(), key=lambda item: item[1], reverse=True))
                i = 0
                d['v0001']['days'][x]['user']['navigation'] = {}
                for k,v in tmpNav.items():
                    i += 1
                    if i >= 24:
                        break
                    d['v0001']['days'][x]['user']['navigation'][k] = v
                del tmpNav
    
    #try:
    #    print("\nsummarizemonthV0001 20250223->user: " + "\n " + str(d['v0001']['days']['20250223']['user']) )
    #except KeyError:
    #    pass   # do nothing
    return d

def compMonthV0001(d):
    # sort accumulated countries              
    for x in d['v0001']['days']:
        if len(x) == 6:
            d['v0001']['days'][x]['user']['countryHits'] = dict(sorted(d['v0001']['days'][x]['user']['countryHits'].items(), key=lambda item: item[1], reverse=True))

        # sort accumulated topurl
        for x in d['v0001']['days']['user']:
            if len(x) == 6:
                 if 'topUrl' in d['v0001']['days'][x]['user']:
                     tmpTopUrl = dict(sorted(d['v0001']['days'][x]['user']['topUrl'].items(), key=lambda item: item[1], reverse=True))
                     i = 0
                     d['v0001']['days'][x]['user']['topUrl'] = {}
                     for k,v in tmpTopUrl.items():
                         i += 1
                         if i >= 24:
                             break
                         d['v0001']['days'][x]['user']['topUrl'][k] = v
                     del tmpTopUrl

    return d
